@@ -2,7 +2,7 @@ package AnyEvent::CouchDB;
 
 use strict;
 use warnings;
-our $VERSION = '1.26';
+our $VERSION = '1.27';
 
 use JSON;
 use AnyEvent::HTTP;
@@ -522,6 +522,10 @@ franck [http://github.com/franckcuny] (for feature enhancements)
 
 Luke Closs (for bug fixes)
 
+Michael Henson (bug fixes)
+
+James Howe (bug reports)
+
 =head1 COPYRIGHT
 
 Copyright (c) 2008-2011 John BEPPU E<lt>beppu@cpan.orgE<gt>.
